# *
# *	Copyright (c) 2000 Alberto Reggiori / <alberto.reggiori@jrc.it>
# *	ISIS/RIT, Joint Research Center Ispra (I)
# *
# * NOTICE
# *
# * This product is distributed under a BSD/ASF like license as described in the 'LICENSE'
# * file you should have received together with this source code. If you did not get a
# * a copy of such a license agreement you can pick up one at:
# *
# *     http://xml.jrc.it/RDFStore/LICENSE
# *
# *
package RDF::Store::FindIndex;
{
use Carp;
use Data::MagicTie;
use RDF::Digest::Util;

sub new {
	my ($pkg,$indexesToTreemapsOfStatements)=@_;
	#k$fullpath,$lr,$mode,$noft,$Q,$dbms_host_or_style,$dbms_port) = @_;

	my $self={};

	if(	(defined $indexesToTreemapsOfStatements) &&
		(ref($indexesToTreemapsOfStatements) =~ /HASH/) ) {
		$self->{_indexesToTreemapsOfStatements} = $indexesToTreemapsOfStatements;
	} else {
		#tie them local/remote otherwise in-memory :-)
		$self->{_indexesToTreemapsOfStatements} = {};

		#warn "TIE disabled for FindIndex!!\n";

		# we do like hash tables :)
                #tie %{$self->{_indexesToTreemapsOfStatements}},
                #               'Data::MagicTie',
                #                $fullpath.'index/triples',
                #                $lr,
		#		$mode,
                #                0, #we do Free/Thaw values 
                #                $Q,
		#		$dbms_host_or_style,$dbms_port
                #        or croak "Cannot tie triples INDEX database ($!)\n";
	};
	
	bless $self,$pkg;
};

# not really needed - we have tied hashes :)
# this means that in-menory and/or persistent indexes are really transparent to the user
sub size {
};

sub addLookup {
	croak "Statement ".$_[1]." is not an instance of RDF::Statement"
                unless( (defined $_[1]) && $_[1]->isa('RDF::Statement'));

	my $subject = $_[1]->subject();
	my $predicate = $_[1]->predicate();
	my $object = $_[1]->object();

	# add look ups here...
	$_[0]->put($_[0]->getLookupValue($subject,undef,undef), $_[1]);
	$_[0]->put($_[0]->getLookupValue(undef,$predicate,undef), $_[1]);
	$_[0]->put($_[0]->getLookupValue(undef,undef,$object), $_[1]);
	$_[0]->put($_[0]->getLookupValue(undef,$predicate,$object), $_[1]);
	$_[0]->put($_[0]->getLookupValue($subject,undef,$object), $_[1]);
	$_[0]->put($_[0]->getLookupValue($subject,$predicate,undef), $_[1]);
	$_[0]->put($_[0]->getLookupValue($subject,$predicate,$object), $_[1]);
};

sub removeLookup {
	croak "Statement ".$_[1]." is not an instance of RDF::Statement"
                unless((defined $_[1]) && ($_[1]->isa('RDF::Statement')));

	my $subject = $_[1]->subject();
	my $predicate = $_[1]->predicate();
	my $object = $_[1]->object();

	#remove it
	$_[0]->remove($_[0]->getLookupValue($subject,undef,undef), $_[1]);
	$_[0]->remove($_[0]->getLookupValue(undef,$predicate,undef), $_[1]);
	$_[0]->remove($_[0]->getLookupValue(undef,undef,$object), $_[1]);
	$_[0]->remove($_[0]->getLookupValue(undef,$predicate,$object), $_[1]);
	$_[0]->remove($_[0]->getLookupValue($subject,undef,$object), $_[1]);
	$_[0]->remove($_[0]->getLookupValue($subject,$predicate,undef), $_[1]);
	$_[0]->remove($_[0]->getLookupValue($subject,$predicate,$object), $_[1]);
};

sub multiget {
	croak "Subject ".$_[1]." is not an instance of RDF::Resource"
                unless((not(defined $_[1])) || ($_[1]->isa('RDF::Resource')));
	croak "Predicate ".$_[2]." is not an instance of RDF::Resource"
                unless((not(defined $_[2])) || ($_[2]->isa('RDF::Resource')));
	croak "Object ".$_[3]." is not an instance of RDF::RDFNode"
                unless((not(defined $_[3])) || ($_[3]->isa('RDF::RDFNode')));

	# return HASH reference, tied/no-tied, local, remote
	$_[0]->{_indexesToTreemapsOfStatements}->{ $_[0]->getLookupValue($_[1],$_[2],$_[3]) }; #FETCH
};

sub put {
	croak "Index ".$_[1]." is not an integer"
		unless(int($_[1]));
	croak "Statement ".$_[2]." is not an instance of RDF::Statement"
                unless((defined $_[2]) && ($_[2]->isa('RDF::Statement')));

	my $stuff = $_[0]->{_indexesToTreemapsOfStatements}->{$_[1]}; #FETCH
	$stuff = {}
		unless( (defined $stuff) &&
			(ref($stuff) =~ /HASH/) );
	#do it
	$stuff->{$_[2]->hashCode()} = $_[2];

	$_[0]->{_indexesToTreemapsOfStatements}->{$_[1]} = $stuff; #STORE
};

sub remove {
	croak "Index ".$_[1]." is not an integer"
		unless(int($_[1]));
	croak "Statement ".$_[2]." is not an instance of RDF::Statement"
                unless((defined $_[2]) && ($_[2]->isa('RDF::Statement')));

	my $stuff = $_[0]->{_indexesToTreemapsOfStatements}->{$_[1]}; #FETCH

	#do it
	delete $stuff->{$_[2]->hashCode()};

	if(scalars(keys %{$stuff})>0) {
		$_[0]->{_indexesToTreemapsOfStatements}->{$_[1]} = $stuff; #STORE
	} else {
		#zap it
		delete $_[0]->{_indexesToTreemapsOfStatements}->{$_[1]};
	};
};

sub getLookupValue {
	croak "Subject ".$_[1]." is not an instance of RDF::Resource"
                unless((not(defined $_[1])) || ($_[1]->isa('RDF::Resource')));
	croak "Predicate ".$_[2]." is not an instance of RDF::Resource"
                unless((not(defined $_[2])) || ($_[2]->isa('RDF::Resource')));
	croak "Object ".$_[3]." is not an instance of RDF::RDFNode"
                unless((not(defined $_[3])) || ($_[3]->isa('RDF::RDFNode')));

	my $s1 = (defined $_[1]) ? $_[1]->hashCode() : 0;
	my $s2 = (defined $_[2]) ? $_[2]->hashCode() : 0;
	my $s3 = (defined $_[3]) ? $_[3]->hashCode() : 0;

	#cool!
  	return int( ($s1 * 37 + $s2) * 37 + $s3 );
};

1;
};

__END__

=head1 NAME

RDF::Store::FindIndex - implementation of the FindIndex RDF API

=head1 SYNOPSIS

	use RDF::Store::FindIndex;
	my $myindex = new RDF::Store::FindIndex({});

	use Data::MagicTie;
	tie %index,"Data::MagicTie",'index';
	my $mypresistentindex = new RDF::Store::FindIndex(\%index);

=head1 DESCRIPTION

This modules implement a very simple and dirty indexing for RDF::Statement(s) using three hashes
one indexing by subject, one by predicate one by object. Each hash contain a copy of the statement
actually containing such a subject, predicate or object. It is not really efficient due the fact that it
requires to keep as value in hashes a copy of the BLOBed statement - see Data::MagicTie(3)

An instance of RDF::Store::FindIndex can be passed to RDF::Store::Model to help it searching the RDF triplets store.

=head1 SEE ALSO

Data::MagicTie(3) RDF::Store::Model(3)

=head1 AUTHOR

	Alberto Reggiori <alberto.reggiori@jrc.it>
